$TITLE ('COS DCE Operate-mode Patchable Constants, version 01.029')
$PAGELENGTH(42)
;*****************************************************************************
; File -- patchcst.a51
; Purpose -- COS DCE 8051 operate code, patchable constants.  In this file are
;            defined and allocated various parameters which control operation
;            of the operate-mode software.  Items which must be initialized to
;            a particular value are declared in CODE space whereas simple
;            working variables are declared in XDATA space.  Since this code
;            will reside at or above address 8000h, the CODE and XDATA space
;            are one and the same so even CODE space items may be changed
;            (i.e., these are the patchable constants).
;
; Author -- William I. Clement, Ph.D., President
;           Clement Engineering, Inc.
;           1261 Dogwood Road
;           Arnold, MD  21012-2220
;           (410) 518-6591
;
; Change History --
; 23Oct00   First Operate-mode version based on Boot-mode verion 01.05
;*****************************************************************************

$SET (PATCHCST=1)
$INCLUDE(common.inc)

;*****************************************************************************
;                         Delta CRC for upper 32KB
;*****************************************************************************
   CSEG   at   0FFFEh
   DW   0D416h              ; Change this with each version.
;-----------------------------------------------------------------------------


   CSEG  at  PATCHCST_SEG   ; This is where we place the patchable constants
                            ; in CODE space.

;*****************************************************************************
;                          Flight Software version
;*****************************************************************************
FILL_TO_BOUNDARY   16, 0FFh
fsw_version   equ   1029
mLFDSWVER:   ; Version number 'fsw_version'
   DB   LOW(fsw_version), HIGH(fsw_version)
;-----------------------------------------------------------------------------

;*****************************************************************************
;                       ADC Conversion delay multiples
;*****************************************************************************
mMUX_DELAY:
   DB   5                   ; Delay (multiples of 0.0105 msec) between setting
                            ;    the MUX and starting the ADC.
mADC_DELAY:
   DB   1                   ; Delay (multiples of 0.0105 msec) between start
                            ;    of ADC and reading the ADC value.
;-----------------------------------------------------------------------------

;*****************************************************************************
; 'HSTERR_TABLE' provides conversion from diagnostic to HST error.
; Entries are pointers to HST error parameter.  Null pointers are for
; diagnostics which are NOT HST errors.
;*****************************************************************************
null_ptr   equ   0

FILL_TO_BOUNDARY   16, 0FFh
HSTERR_TABLE:
   DW   null_ptr,     PARAM2,   mLFMXFER,     PARAM2     ; 00..03
   DW    SERIALN,    SERIALN,    SERIALN,   null_ptr     ; 04..07
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 08..0B
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 0C..0F

   DW   null_ptr,       RS17,   null_ptr,    SERIALN     ; 10..13
   DW   null_ptr,   null_ptr,   null_ptr,   mLFCTIME     ; 14..17
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 18..1B
   DW   null_ptr,   null_ptr,   null_ptr,   mLFCTIME     ; 1C..1F

   DW   null_ptr,   null_ptr,    mBADCRP,   null_ptr     ; 20..23
   DW   null_ptr,   null_ptr,   mBADAUXI,   null_ptr     ; 24..27
   DW   null_ptr,   null_ptr,    ptr_hva,    ptr_hvb     ; 28..2B
   DW   null_ptr,   null_ptr,    ptr_aux,   null_ptr     ; 2C..2F

   DW   null_ptr,    mBADISR, mBADREGION,      mVERR     ; 30..33
   DW      mVERR,      mVERR,      mVERR,   null_ptr     ; 34..37
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 38..3B
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 3C..3F

   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 40..43
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 44..47
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 48..4B
   DW   null_ptr,   null_ptr,   null_ptr,   null_ptr     ; 4C..4F
;-----------------------------------------------------------------------------

;*****************************************************************************
; 'CRC_REGIONS' is an array of 12-byte structures used to implement
; background CRC checking over multiple regions of memory.  The contents of
; each element of the array are as follows:
;    (1)  Starting address in memory (16 bits, big-endian)
;    (2)  Number of bytes to check (16 bits, big-endian)
;    (3)  Memory space (0=XDATA, 1=CODE) (8-bits)
;    (4)  Next region number (8-bits)
;    (5)  CRC for this region (16-bits)
;    (6)  Previous CRC for this region, if non-zero (16-bits)
;    (7)  Diagnostic to send if CRCs miscompare, if MSB=5Ah (16-bits)
; Region 0 is the CRC determined by LFDCRC command ('LFMCRC')
; Region 1 is the CRC of the PROM (first 6KB). ('LFMROM')
; Region 2 is the CRC of the remain 2KB of PROM.
; Regions 3-9 are various blocks in the operate code area (above 32 KB).
; Regions 10-15 are for expansion.
; NOTE:  The labels at the beginning and end of this code segment, 
;    'CRC_REGIONS' and 'end_CRC_REGIONS' define the maximum number
;    of regions allocated and, hence, allowed ('num_regions').
;    The software will check to ensure that no unauthorized region is
;    requested since this would lead to software 'scribbling' in memory.
; NOTE:  To halt background CRC calculations, merely make a region point to
;    a 'next region' beyond the allowable range (i.e. >= 'num_regions')
; NOTE:  The command 'DCE_LFDCRC' will populate Region 0 with data specific
;    to its assigned task and zero out the 'previous CRC' to prevent
;    diagnostic and/or HST error.  It will NOT change the 'next region'
;    specifier.  That is up to the operator to command such a change.
;    By default, the 'next region' is region 1.
; CRCs for regions 1-9 for operate build v01.028 and higher are ...
;    Address Range    CRC
;    =============    ======
;    0x0000-0x17FF    0xC001    ; boot v01.13
;    0x1800-0x1FFF    0xC001    ; ditto
;    0x8000-0x84FF    0xC0DE
;    0x8500-0x89FF    0xC0DE
;    0x8A00-0x8FFF    0xC0DE
;    0x9000-0x99FF    0xC0DE
;    0x9A00-0x9DFF    0xC0DE
;    0x9E00-0xA3FF    0xC0DE
;    0xA400-0xADFF    0xC0DE
;*****************************************************************************
FILL_TO_BOUNDARY   16, 0FFh
CRC_REGIONS:

; Region 0 (used by LFDCRC for its CRC region) ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 1             ; 0=XDATA space, next region is '1'
mLFMCRC:                 ; CRC mnemonic LFMCRC
mLFMCRC00:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
CRC_REGIONS_00:          ; label used for computing 'sz_regions'

; Region 1 (PROM, 6KB) ...
   DW   00000h, 01800h   ; starting address, number of bytes
   DB   1, 2             ; 1=CODE space, next region
mLFMROM:                 ; CRC mnemonic LFMROM
mLFMCRC01:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 2 (rest of PROM, 2KB) ...
   DW   01800h, 0800h    ; starting address, number of bytes
   DB   1, 3             ; 1=CODE space, next region
mLFMCRC02:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 3 ...
   DW   reg3, reg3len    ; starting address, number of bytes
   DB   0, 4             ; 0=XDATA space, next region
mLFMCRC03:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 4 ...
   DW   reg4, reg4len    ; starting address, number of bytes
   DB   0, 5             ; 0=XDATA space, next region
mLFMCRC04:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 5 ...
   DW   reg5, reg5len    ; starting address, number of bytes
   DB   0, 6             ; 0=XDATA space, next region
mLFMCRC05:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 6 ...
   DW   reg6, reg6len    ; starting address, number of bytes
   DB   0, 7             ; 0=XDATA space, next region
mLFMCRC06:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 7 ...
   DW   reg7, reg7len    ; starting address, number of bytes
   DB   0, 8             ; 0=XDATA space, next region
mLFMCRC07:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 8 ...
   DW   reg8, reg8len    ; starting address, number of bytes
   DB   0, 9             ; 0=XDATA space, next region
mLFMCRC08:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 9 ...
   DW   reg9, reg9len    ; starting address, number of bytes
   DB   0, 1             ; 0=XDATA space, next region
mLFMCRC09:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 10 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region
mLFMCRC10:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 11 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region (invalid)
mLFMCRC11:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 12 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region (invalid)
mLFMCRC12:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 13 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region (invalid)
mLFMCRC13:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 14 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region (invalid)
mLFMCRC14:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC
   
; Region 15 ...
   DW   00000h, 00000h   ; starting address, number of bytes
   DB   0, 255           ; 0=XDATA space, next region (invalid)
mLFMCRC15:
   DW   0, 0             ; calculated CRC, previous CRC
   DW   DIAG0032         ; Diagnostic:  Bad CRC

end_CRC_REGIONS:

; The software uses 'num_regions' to limit the allowed values for the
; 'next region' parameter. 'sz_regions' is the number of bytes used by
; each entry in the above table ...
sz_regions    equ   (CRC_REGIONS_00 - CRC_REGIONS)
num_regions   equ   (end_CRC_REGIONS - CRC_REGIONS)/sz_regions
;-----------------------------------------------------------------------------

;*****************************************************************************
; The Current-limit activity uses a structure which holds all parameters and
; variables used by the task.  There are three such structures -- one for
; HV-A, one for HV-B, and one for the Auxillary Power Supply.  The last two
; parameters in each structure are pointers to buffers -- the circular history
; buffer and the histogram buffer.  Those buffers are located in XDATA space.
;*****************************************************************************
; High-Voltage, Segment A ...
ilim_param_hva:
   DB   128                 ; LFHVILIM (copy) -- current limit
   DB   5                   ; Over-current count limit
   DB   0                   ; Over-current counts
   DB   0                   ; present reading
   DB   0                   ; previous reading
ptr_hva:
   DW   0                   ; offset into circular buffer (little-endian)
   DW   mHV_IMON_BUFA       ; pointer to circular buffer (big-endian)
   DW   mHV_IMON_PDFA       ; pointer to histogram (big-endian)

; High-Voltage, Segment B ...
ilim_param_hvb:
   DB   128                 ; LFHVILIM (copy) -- current limit
   DB   5                   ; Over-current count limit
   DB   0                   ; Over-current counts
   DB   0                   ; present reading
   DB   0                   ; previous reading
ptr_hvb:
   DW   0                   ; offset into circular buffer (little-endian)
   DW   mHV_IMON_BUFB       ; pointer to circular buffer (big-endian)
   DW   mHV_IMON_PDFB       ; pointer to histogram (big-endian)

; Low-Voltage Auxillary supply ...
ilim_param_aux:
mLFRILIM:
   DB   128                 ; LFRILIM -- current limit
   DB   62                  ; Over-current count limit
   DB   0                   ; Over-current counts
   DB   0                   ; present reading
   DB   0                   ; previous reading
ptr_aux:
   DW   0                   ; offset into circular buffer (little-endian)
   DW   mLV_IMON_BUF        ; pointer to circular buffer (big-endian)
   DW   mLV_IMON_PDF        ; pointer to histogram (big-endian)

ilim_param_all:
   DB   20                  ; number of periods after shutdown to continue
                            ;    collecting HV and Aux current data.

mLFHVILIM:
   DB    128                ; HV current limit (copied to segment A and B)

;-----------------------------------------------------------------------------

;*****************************************************************************
; The Count-Rate Protection (CRP) activity uses a structure which holds all
; parameters and variables used by the activity.  There are two such
; structures -- Segment A and Segment B.  The last parameter in the structure
; is a pointer to the circular buffer which holds the last 'T' count rates
; (collected once per second).  That buffer is located in XDATA space.
;*****************************************************************************
; Segment A ...
crp_segA:
mLFPINTA:
   DB   0                   ; LFPINTA
mLFPCNTA:
   DB   0FFh, 0FFh          ; LFPCNTA (little-endian)
   DB   0                   ; offset into circular buffer
   DW   mCRP_BUFA           ; pointer to circular buffer (big-endian)

; Segment B ...
crp_segB:
mLFPINTB:
   DB   0                   ; LFPINTB
mLFPCNTB:
   DB   0FFh, 0FFh          ; LFPCNTB (little-endian)
   DB   0                   ; offset into circular buffer
   DW   mCRP_BUFB           ; pointer to circular buffer (big-endian)
;-----------------------------------------------------------------------------

;*****************************************************************************
; The High-Voltage Ramping activity uses a structure which holds all
; parameters and variables used by the activity.  There are two such
; structures -- Segment A and Segment B.  Additionally, HV ramping on both
; segments share some common parameters and variables.  These are found in
; the 'hv_segAB' structure, below.
;*****************************************************************************
; Segment A ...
hv_segA:
mLFHVLOA:
   DB   0                   ; LFHVLOA  -- low voltage
mLFHVNOMA:
   DB   0                   ; LFHVNOMA -- nominal voltage
mLFHVMAXA:
   DB   0                   ; LFHVMAXA -- holds value written to VMAX register
mLFHVTGTA:
   DB   0                   ; LFHVTGTA -- target voltage
mLFHVDACA:
   DB   0                   ; LFHVDACA -- holds value written to DAC (VSET)
   DW   0                   ; count-up to next ramp step (little-endian)

; Segment B ...
hv_segB:
mLFHVLOB:
   DB   0                   ; LFHVLOB
mLFHVNOMB:
   DB   0                   ; LFHVNOMB
mLFHVMAXB:
   DB   0                   ; LFHVMAXB
mLFHVTGTB:
   DB   0                   ; LFHVTGTB
mLFHVDACB:
   DB   0                   ; LFHVDACB
   DW   0                   ; count-up to next ramp step (little-endian)

; Segment A & B ...
hv_segAB:
mLFHSTATE:
   DB   0                   ; LFHSTATE -- HV state variable
mLFHRAMPT:
   ; LFHRAMPT (little-endian) -- seconds between ramp steps ...
   DB   LOW(0Ah), HIGH(0Ah)
;-----------------------------------------------------------------------------

;*****************************************************************************
; Door Activity.
;*****************************************************************************
DOOR_TIMEOUT:   DB   210    ; timeout, seconds
;-----------------------------------------------------------------------------

;*****************************************************************************
; The housekeeping script is composed of 4-byte entries.  The first byte is
; the script opcode and the last byte is the repetition count -- how many
; successive entities of the same type (same opcode) to collect and put
; into housekeeping (also at successive addresses).  The opcodes (first byte)
; are:
;        00h -- End of script
;        01h -- Read bytes from CODE space
;        02h -- Read bytes from XDATA space (or memory-mapped I/O)
;        03h -- Read bytes from DATA space
;        06h -- Read bytes from Analog I/O
;        07h -- Move housekeeping pointer to specified address
;        08h -- Skip 'N' bytes of housekeeping packet
;        09h -- Clear 'N' bytes of housekeeping packet
;        46h -- Read TDC 'A'
;        82h -- Read TDC 'B'
;
; Depending on script opcode, bytes 2 & 3 take on various meanings.  They are:
;     Opcode -- Bytes 2&3
;        01h -- 16-bit address (MSB first)
;        02h -- 16-bit address (MSB first)
;        03h -- 16-bit address (MSB first, ignored)
;        06h -- 8-bit MUX address, N/A
;        07h -- 16-bit address to which to move HKP pointer
;        08h -- N/A
;        09h -- N/A
;        46h -- 8-bit TDC MUX address, N/A
;        82h -- 8-bit TDC MUX address, N/A
;
; Byte 4 is always the repetition count.
;*****************************************************************************

; The housekeeping script for Operate mode ...
FILL_TO_BOUNDARY   16, 0FFh
hkp_script:
   DB   op_xdata,  HIGH(PH_A),     LOW(PH_A),     128 ; PH 'A' (first 64)
   DB   op_xdata,  HIGH(PH_A+128), LOW(PH_A+128), 128 ; PH 'A' (last 64)
   DB   op_xdata,  HIGH(PH_B),     LOW(PH_B),     128 ; PH 'B' (first 64)
   DB   op_xdata,  HIGH(PH_B+128), LOW(PH_B+128), 128 ; PH 'B' (last 64)
   DB   op_xdata,  HIGH(mLFDERR),  LOW(mLFDERR),   8 ; LFDERR (8 bytes)
   DB   op_xdata,  HIGH(mLFDERRP), LOW(mLFDERRP), 16 ; LFDERRP (16 bytes)
   DB   op_jump,   HIGH(1664h),    LOW(1664h),     0 ; jump
   ;
   DB   op_xdata,  HIGH(RS17),     LOW(RS17),     28 ; LFDCBUF
   DB   op_xdata,  HIGH(mLFCTIME), LOW(mLFCTIME),  4 ; LFCTIME
   DB   op_jump,   HIGH(1694h),    LOW(1694h),     0 ; jump
   ;
   DB   op_xdata,  HIGH(mDEC_A), LOW(mDEC_A),  4 ; DEC 'A'
   DB   op_xdata,  HIGH(mDEC_B), LOW(mDEC_B),  4 ; DEC 'B'
   DB   op_jump,   HIGH(16A4h),  LOW(16A4h),   0 ; jump
   ;
   DB   op_xdata,  HIGH(mSDC_1), LOW(mSDC_1),  4 ; SDC '1'
   DB   op_xdata,  HIGH(mSDC_2), LOW(mSDC_2),  4 ; SDC '2'
   DB   op_xdata,  HIGH(mFEC_A), LOW(mFEC_A),  4 ; FEC 'A'
   DB   op_xdata,  HIGH(mFEC_B), LOW(mFEC_B),  4 ; FEC 'B'
   DB   op_jump,   HIGH(16BCh),  LOW(16BCh),   0 ; jump
   ;
   DB   op_TDC_A,  40h,  0,  2  ; LFGSHFAX, TDC_A MUX=40h
                                ; LFGSTRAX, TDC_A MUX=41h
   DB   op_TDC_B,  40h,  0,  2  ; LFGSHFBX, TDC_B MUX=40h
                                ; LFGSTRBX, TDC_B MUX=41h
   DB   op_a2d,    08h,  0,  4  ; LFTDVAA, MUX=08h (1)
                                ; LFTACT,  MUX=09h (1)
                                ; LFTHVFM, MUX=0Ah (1)
                                ; LFTHVPS, MUX=0Bh (1)
   DB   op_a2d,    0Dh,  0,  1  ; LFTIP,   MUX=0Dh
   DB   op_a2d,    11h,  0,  1  ; LFTDVAB, MUX=11h
   DB   op_a2d,    0Eh,  0,  2  ; LFTAMPA, MUX=0Eh (1)
                                ; LFTAMPB, MUX=0Fh (1)
   DB   op_a2d,    0Ch,  0,  1  ; LFTTDCA, MUX=0Ch
   DB   op_a2d,    10h,  0,  1  ; LFTTDCB, MUX=10h
   DB   op_a2d,    12h,  0,  4  ; LFTLVPC, MUX=12h (1)
                                ; LFTDCE,  MUX=13h (1)
                                ; LFTFAN,  MUX=14h (1)
                                ; LFVP5DC, MUX=15h (1)
   DB   op_jump,   HIGH(16D2h),   LOW(16D2h),    0  ; jump
   ;
   DB   op_a2d,    22h,  0,  6  ; LFHIMONA, MUX=22h (1)
                                ; LFHIMONB, MUX=23h (1)
                                ; LFHVMONA, MUX=24h (1)
                                ; LFHVMONB, MUX=25h (1)
                                ; LFHQANA,  MUX=26h (1)
                                ; LFHQANB,  MUX=27h (1)
   DB   op_TDC_A,  50h,  0,  1  ; LFVP15TA, TDC_A MUX=50h
   DB   op_TDC_B,  50h,  0,  1  ; LFVP15TB, TDC_B MUX=50h
   DB   op_TDC_A,  52h,  0,  1  ; LFVP5TA,  TDC_A MUX=52h
   DB   op_TDC_B,  52h,  0,  1  ; LFVP5TB,  TDC_B MUX=52h
   DB   op_TDC_A,  51h,  0,  1  ; LFVM15TA, TDC_A MUX=51h
   DB   op_TDC_B,  51h,  0,  1  ; LFVM15TB, TDC_B MUX=51h
   DB   op_TDC_A,  53h,  0,  1  ; LFVM5TA,  TDC_A MUX=53h
   DB   op_TDC_B,  53h,  0,  1  ; LFVM5TB,  TDC_B MUX=53h
   DB   op_a2d,    16h,  0,  2  ; LFVP15D,  MUX=16h (1)
                                ; LFVM15D,  MUX=17h (1)
   DB   op_a2d,    44h,  0,  2  ; LFHVSETA, MUX=44h (1)
                                ; LFHVSETB, MUX=45h (1)
   DB   op_xdata,  HIGH(mLFSBITS2), LOW(mLFSBITS2), 2  ; 'Status bits 2'
   DB   op_TDC_A,  46h,  0,  1  ; LFGUQTA,  TDC_A MUX=46h
   DB   op_TDC_B,  46h,  0,  1  ; LFGUQTB,  TDC_B MUX=46h
   DB   op_TDC_A,  42h,  0,  1  ; LFGBWKAX, TDC_A MUX=42h
   DB   op_TDC_B,  42h,  0,  1  ; LFGBWKBX, TDC_B MUX=42h
   DB   op_TDC_A,  43h,  0,  1  ; LFGEWKAX, TDC_A MUX=43h
   DB   op_TDC_B,  43h,  0,  1  ; LFGEWKBX, TDC_B MUX=43h
   DB   op_jump,   HIGH(16EEh),  LOW(16EEh),     0  ; jump
   ;
   DB   op_TDC_A,  45h,  0,  1  ; LFGLQTA,  TDC_A MUX=45h
   DB   op_TDC_B,  45h,  0,  1  ; LFGLQTB,  TDC_B MUX=45h
   DB   op_TDC_A,  47h,  0,  1  ; LFGTTAX,  TDC_A MUX=47h
   DB   op_TDC_B,  47h,  0,  1  ; LFGTTBX,  TDC_B MUX=47h
   DB   op_xdata,  HIGH(mLFSBITS0), LOW(mLFSBITS0), 2  ; 'Status bits 0'
   DB   op_xdata,  HIGH(mLFSBITS1), LOW(mLFSBITS1), 2  ; 'Status bits 1'
   DB   op_jump,   HIGH(16FCh),     LOW(16FCh),     0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFMROM),   LOW(mLFMROM),   2  ; LFMROM
   DB   op_xdata,  HIGH(mLFDSWVER), LOW(mLFDSWVER), 2  ; LFDSWVER
   DB   op_xdata,  HIGH(mLFCPKTL),  LOW(mLFCPKTL),  4  ; LFCPKT
   DB   op_xdata,  HIGH(mLFDLSTC),  LOW(mLFDLSTC),  1  ; LFDLSTC
   DB   op_xdata,  HIGH(mLFHSTATE), LOW(mLFHSTATE), 1  ; LFHSTATE
   DB   op_TDC_A,  54h,  0,  1  ; LFGREFA,  TDC_A MUX=54h
   DB   op_TDC_B,  54h,  0,  1  ; LFGREFB,  TDC_B MUX=54h
   DB   op_xdata,  HIGH(tLFDDIAGS+0), LOW(tLFDDIAGS+0), 1  ; LFDDIAG0
   DB   op_xdata,  HIGH(tLFDDIAGS+2), LOW(tLFDDIAGS+2), 1  ; LFDDIAG1
   DB   op_xdata,  HIGH(mLFRILIM), LOW(mLFRILIM), 1  ; LFRILIM
   DB   op_xdata,  HIGH(mLFHVILIM), LOW(mLFHVILIM), 1  ; LFHVILIM
   DB   op_xdata,  HIGH(mLFDCMDX), LOW(mLFDCMDX), 2 ; LFDCMDX
   DB   op_xdata,  HIGH(mLFPCNTA), LOW(mLFPCNTA), 2 ; LFPCNTA
   DB   op_a2d,    43h,  0,  1  ; LFVPMON,  MUX=43h
   DB   op_xdata,  HIGH(tLFDDIAGS+4), LOW(tLFDDIAGS+4), 1  ; LFDDIAG2
   DB   op_jump,   HIGH(1714h),       LOW(1714h),       0  ; jump
   ;
   DB   op_a2d,    42h,  0,  1  ; LFRACTAN, MUX=42h
   DB   op_a2d,    41h,  0,  1  ; LFRMTRAN, MUX=41h
   DB   op_a2d,    21h,  0,  1  ; LFRAXI,   MUX=21h
   DB   op_a2d,    40h,  0,  1  ; LFRAXAN,  MUX=40h
   DB   op_xdata,  HIGH(mLFDCMDR), LOW(mLFDCMDR), 2 ; LFDCMDR
   DB   op_xdata,  HIGH(mLFMCRC), LOW(mLFMCRC),   2 ; LFDCRC
   DB   op_xdata,  HIGH(mLFPINTA), LOW(mLFPINTA), 1 ; LFPINTA
   DB   op_jump,   HIGH(171Eh),       LOW(171Eh), 0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFPCNTB), LOW(mLFPCNTB), 2 ; LFPCNTB
   DB   op_xdata,  HIGH(mLFHVLOA), LOW(mLFHVLOA), 1 ; LFHVLOA
   DB   op_xdata,  HIGH(mLFHVLOB), LOW(mLFHVLOB), 1 ; LFHVLOB
   DB   op_jump,   HIGH(1724h),   LOW(1724h),     0 ; jump
   ;
   DB   op_xdata,  HIGH(mLFHVNOMA), LOW(mLFHVNOMA), 1 ; LFHVNOMA
   DB   op_xdata,  HIGH(mLFHVNOMB), LOW(mLFHVNOMB), 1 ; LFHVNOMB
   DB   op_a2d,    84h,  0,  2  ; LFHVMAXA, MUX=84h
                                ; LFHVMAXB, MUX=85h
   DB   op_xdata,  HIGH(mLFPINTB), LOW(mLFPINTB), 1 ; LFPINTB
   DB   op_jump,   HIGH(172Ah),       LOW(172Ah), 0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFSBITS3), LOW(mLFSBITS3), 2  ; 'Status bits 3'
   DB   op_xdata,  HIGH(mLFSBITS4), LOW(mLFSBITS4), 1  ; 'Status bits 4'
   DB   op_xdata,  HIGH(mLFSBITS5), LOW(mLFSBITS5), 1  ; 'Status bits 5'
   DB   op_xdata,  HIGH(mLFGSTIMA), LOW(mLFGSTIMA), 1  ; LFGSTIMA
   DB   op_xdata,  HIGH(mLFGSTIMB), LOW(mLFGSTIMB), 1  ; LFGSTIMB
   DB   op_xdata,  HIGH(mLFDPERF), LOW(mLFDPERF), 2 ; LFDPERF
   DB   op_jump,   HIGH(1735h),   LOW(1735h),     0 ; jump
   ;
   DB   op_a2d,    20h,  0,  1  ; LFRPOS, MUX=20h
   DB   op_xdata,  HIGH(mLFDMEXTR), LOW(mLFDMEXTR), 1  ; LFDMEXTR
   DB   op_xdata,  HIGH(mLFDMDATA), LOW(mLFDMDATA), 1  ; LFDMDATA
   DB   op_xdata,  HIGH(mLFDMONS),  LOW(mLFDMONS),  8  ; LFDMONS
   DB   op_xdata,  HIGH(mLFDMADD),  LOW(mLFDMADD), 16  ; LFDMADD
   DB   op_TDC_A,  62h,  0,  2  ; LFGBWKAY, TDC_A MUX=62h
                                ; LFGEWKAY, TDC_A MUX=63h
   DB   op_TDC_B,  62h,  0,  2  ; LFGBWKBY, TDC_B MUX=62h
                                ; LFGEWKBY, TDC_B MUX=63h
   DB   op_data, 0, LOW(DOOR_TIMER), 1  ; LFRTIMER
   DB   op_jump,   HIGH(175Eh),     LOW(175Eh),     0  ; jump
   ;
   DB   op_TDC_A,  44h,  0,  1  ; LFGTTAY,  TDC_A MUX=44h
   DB   op_TDC_B,  44h,  0,  1  ; LFGTTBY,  TDC_B MUX=44h
   DB   op_TDC_A,  60h,  0,  2  ; LFGSHFAY, TDC_A MUX=60h
                                ; LFGSTRAY, TDC_A MUX=61h
   DB   op_TDC_B,  60h,  0,  2  ; LFGSHFBY, TDC_B MUX=60h
                                ; LFGSTRBY, TDC_B MUX=61h
   DB   op_jump,   HIGH(1774h),     LOW(1774h),     0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFHRAMPT), LOW(mLFHRAMPT), 1  ; LFHRAMPT
   DB   op_jump,   HIGH(1776h),     LOW(1776h),     0  ; jump
   ;
   DB   op_xdata,  HIGH(tLFDDIAGS+6), LOW(tLFDDIAGS+6), 1 ; LFDDIAG3
   DB   op_xdata,  HIGH(tLFDDIAGS+8), LOW(tLFDDIAGS+8), 1 ; LFDDIAG4
   DB   op_jump,   HIGH(1780h),   LOW(1780h),    0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFDDIAGS), LOW(mLFDDIAGS), 64  ; LFDDIAGS
   DB   op_a2d,    80h,  0,  2  ; LFVP5DA,  MUX=80h
                                ; LFVP5DB,  MUX=81h
   DB   op_jump,   HIGH(17D2h),   LOW(17D2h),    0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFHVTGTA), LOW(mLFHVTGTA), 1  ; LFHVTGTA
   DB   op_xdata,  HIGH(mLFHVTGTB), LOW(mLFHVTGTB), 1  ; LFHVTGTB

   ; Items added for version 01.06 ...
   ; 16 CRCs ...
   DB   op_xdata,  HIGH(mLFMCRC00), LOW(mLFMCRC00),  2
   DB   op_xdata,  HIGH(mLFMCRC01), LOW(mLFMCRC01),  2
   DB   op_xdata,  HIGH(mLFMCRC02), LOW(mLFMCRC02),  2
   DB   op_xdata,  HIGH(mLFMCRC03), LOW(mLFMCRC03),  2
   DB   op_xdata,  HIGH(mLFMCRC04), LOW(mLFMCRC04),  2
   DB   op_xdata,  HIGH(mLFMCRC05), LOW(mLFMCRC05),  2
   DB   op_xdata,  HIGH(mLFMCRC06), LOW(mLFMCRC06),  2
   DB   op_xdata,  HIGH(mLFMCRC07), LOW(mLFMCRC07),  2
   DB   op_xdata,  HIGH(mLFMCRC08), LOW(mLFMCRC08),  2
   DB   op_xdata,  HIGH(mLFMCRC09), LOW(mLFMCRC09),  2
   DB   op_xdata,  HIGH(mLFMCRC10), LOW(mLFMCRC10),  2
   DB   op_xdata,  HIGH(mLFMCRC11), LOW(mLFMCRC11),  2
   DB   op_xdata,  HIGH(mLFMCRC12), LOW(mLFMCRC12),  2
   DB   op_xdata,  HIGH(mLFMCRC13), LOW(mLFMCRC13),  2
   DB   op_xdata,  HIGH(mLFMCRC14), LOW(mLFMCRC14),  2
   DB   op_xdata,  HIGH(mLFMCRC15), LOW(mLFMCRC15),  2
   ;
   DB   op_data, 0, LOW(maxSP), 1  ; maximum stack pointer
   ;
   DB   op_xdata,  HIGH(mLFMXFER), LOW(mLFMXFER),  2  ; LFMXFER
   ; end of items added for version 01.06
   DB   op_jump,   HIGH(1830h),     LOW(1830h),      0  ; jump
   ;
   DB   op_xdata,  HIGH(mLFDDETID), LOW(mLFDDETID),  1  ; LFDDETID
   ;
   ; The rest is blank -- leave it so.
   ;
   DB   op_end,    0,0,0 ; end of script

   ; Save room for future growth of the script ...
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
   DW   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
;-----------------------------------------------------------------------------

;*****************************************************************************
;                              Scratchpad space
;*****************************************************************************
   XSEG   at   SCRATCH_SEG

start_scratch_seg:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Timer (KEEP IN THIS ORDER) ...
mTICKS:      DS    1        ; counts 20 msec intervals (1/50 second)
mLFCTIME:    DS    4        ; seconds counter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Various others (need not be kept in order) ...

; Diagnostics and HST errors ...
mLFDDIAGS:   DS   64        ; Push-down stack of last 32 diagnostic msgs.
mDIAG_NUM:   DS    1        ; counter of total number of diagnostics
mLFDERR:     DS    8        ; 8 HST error locations
mLFDERRP:    DS   16        ; 8 HST error parameters

; Create an array which holds up to 5 diagnostics stored since the last
; housekeeping packet was sent ...
tLFDDIAGS    equ   $        ; assembler need a little help with its math
                            ; so create this 'equate' at the same address as
                            ; 'mCURR_5'
mCURR_5:     DS   10        ; current (this 1-second) 5 diagnostics w/ serial
mCURR_NUM:   DS    1        ; number of diagnostics in 'mCURR_5' list

; Various ...
mLFCPKTL:    DS    4        ; packet counter (long word)
mLFDCMDR:    DS    2        ; commands received counter
mLFDCMDX:    DS    2        ; commands executed counter
mLFDPERF:    DS    2        ; DCE Performance monitor.  Exec. loop counter.
mLFMXFER:    DS    2        ; upload/download packet CRC
mLFDLSTC:    DS    1        ; last DCE Op-code executed
mCWNS:       DS    2        ; counts commands words received but not stored
mWNS:        DS    4        ; command long word received but not stored
mLFDDETID:   DS    1        ; Detector ID bits (concatenate LFDDETID1:LFDDETID1)
mBADISR:     DS    2        ; 2 bytes for diagnostic 31h
mBADREGION:  DS    2        ; 2-byte parameter for HST error for bad CRC
mBADCRP:     DS    2        ; points to start of circular buffer for the
                            ; which caused shutdown due to CRP violation
mBADAUXI:    DS    2        ; current reading which caused shutdown in the
                            ; door activity

; Memory Monitor parameters ...
mLFDMEXTR:   DS    1        ; external/internal select bits
mLFDMDATA:   DS    1        ; data/code select bits
mLFDMONS:    DS    8        ; data values
mLFDMADD:    DS   16        ; addresses

; 6 status words (first 4 are 16-bits each, last 2 are 8-bits) ...
mLFSBITS0:   DS    2
mLFSBITS1:   DS    2
mLFSBITS2:   DS    2
mLFSBITS3:   DS    2
mLFSBITS4:   DS    1
mLFSBITS5:   DS    1

; Additional Operate-mode variables ...
mLFGSTIMA:   DS    1        ; Stim Pulse Rate for Segment A
mLFGSTIMB:   DS    1        ; Stim Pulse Rate for Segment B

mVERR:       DS    2        ; VMAX and VTGT settings for checking

; Event counters ...
mFEC_A:      DS    4        ; Front-end Counter, Segment A
mDEC_A:      DS    4
mSDC_1:      DS    4
mFEC_B:      DS    4        ; Front-end Counter, Segment B
mDEC_B:      DS    4
mSDC_2:      DS    4

length_scratch_seg   equ   ($-start_scratch_seg)
;-----------------------------------------------------------------------------

;*****************************************************************************
;                               History buffers
;*****************************************************************************
   XSEG  at  HISTORY_SEG

start_history_seg:
; Current-Limit Activity ...
mHV_IMON_BUFA:  DS  1024    ; circular buffer for HV A current readings
mHV_IMON_PDFA:  DS  512     ; PDF (Histogram) of HV A current readings

mHV_IMON_BUFB:  DS  1024    ; circular buffer for HV B current readings
mHV_IMON_PDFB:  DS  512     ; PDF (Histogram) of HV B current readings

mLV_IMON_BUF:   DS  1024    ; circular buffer holding LV current readings
mLV_IMON_PDF:   DS  512     ; PDF (Histogram) of LV current readings

; CRP Activity ...
mCRP_BUFA:      DS  3*256   ; up to 256 3-byte entries -- CRP Segment A
mCRP_BUFB:      DS  3*256   ; up to 256 3-byte entries -- CRP Segment B

length_history_seg   equ   ($-start_history_seg)
;-----------------------------------------------------------------------------

;*****************************************************************************
;                      Reset pattern (6 bytes) at 3FFAh
;*****************************************************************************
   XSEG   at   3FFAh
mPATTERN:    DS    6        ; 6-byte pattern -- watchdog vs. power-on reset
;-----------------------------------------------------------------------------

;*****************************************************************************
;                                End-of-XDATA
;*****************************************************************************
   XSEG  at  0FFFEh
DONT_USE:       DS  2       ; This will generate a linker error if we
                            ; extend XDATA beyond the end of physical memory.
                            ; It also protects the 2 bytes set aside for the
                            ; delta CRC.
;-----------------------------------------------------------------------------

END                         ; patchcst.a51

